/*
 ===============================================================================
 Name        : adc810.c
 Author      : $(author)
 Version     :
 Copyright   : $(copyright)
 Description : main definition
 ===============================================================================
 */

#if defined (__USE_LPCOPEN)
#if defined(NO_BOARD_LIB)
#include "chip.h"
#else
#include "board.h"
#endif
#endif

#include <cr_section_macros.h>

// TODO: insert other include files here
#include "i2cs.h"

// TODO: insert other definitions and declarations here
#define I2C_ADDR 0x25

#define ADC_PSCALE 12
#define ADC_FSCALE 3194
#define ADC_VLADDR 15

void SCT_IRQHandler(void) {
	static uint32_t old_val;
	uint32_t adc_val;
	uint32_t delta;

	if((LPC_SCT->EVFLAG & 1) == 0)
		return;

	delta = LPC_SCT->COUNT_L;
	adc_val = ADC_FSCALE - ((0x10000 + delta - old_val) & 0xffff);
	old_val = delta;
	__disable_irq();
	i2csTxBuf[0] = adc_val >> 8;
	i2csTxBuf[1] = adc_val & 0xff;
	__enable_irq();

	Chip_SCT_ClearEventFlag(LPC_SCT, SCT_EVT_0);
}

int main(void) {

#if defined (__USE_LPCOPEN)
#if !defined(NO_BOARD_LIB)
	// Read clock settings and update SystemCoreClock variable
	SystemCoreClockUpdate();
	// Set up and initialize all required blocks and
	// functions related to the board hardware
	Board_Init();
	// Set the LED to the state of "On"
	Board_LED_Set(0, true);
#endif
#endif

	// TODO: insert code here
	SystemCoreClockUpdate();

	Chip_Clock_EnablePeriphClock(SYSCTL_CLOCK_IOCON);
	Chip_IOCON_PinSetMode(LPC_IOCON, IOCON_PIO0, PIN_MODE_INACTIVE);
	Chip_IOCON_PinSetMode(LPC_IOCON, IOCON_PIO3, PIN_MODE_INACTIVE);
	Chip_Clock_DisablePeriphClock(SYSCTL_CLOCK_IOCON);

	Chip_Clock_EnablePeriphClock(SYSCTL_CLOCK_SWM);
	Chip_SWM_DisableFixedPin(SWM_FIXED_SWCLK);
	Chip_SWM_DisableFixedPin(SWM_FIXED_SWDIO);
	Chip_SWM_EnableFixedPin(SWM_FIXED_ACMP_I1);
	Chip_SWM_MovablePinAssign(SWM_ACMP_O_O, 3);
	Chip_SWM_MovablePinAssign(SWM_CTIN_0_I, 3);
	Chip_SWM_MovablePinAssign(SWM_CTOUT_0_O, 2);
	Chip_SWM_MovablePinAssign(SWM_I2C_SDA_IO, 4);
	Chip_SWM_MovablePinAssign(SWM_I2C_SCL_IO, 1);
	Chip_Clock_DisablePeriphClock(SYSCTL_CLOCK_SWM);

	Chip_SCT_Init(LPC_SCT);
	LPC_SCT->MATCH[0].H = ADC_FSCALE - 1;
	LPC_SCT->MATCHREL[0].H = ADC_FSCALE - 1;
	LPC_SCT->EV[0].CTRL = 0x1010;
	LPC_SCT->EV[0].STATE = 3;
	LPC_SCT->LIMIT_H = 1;
	Chip_SCT_EnableEventInt(LPC_SCT, SCT_EVT_0);

	LPC_SCT->EV[1].CTRL = 0x2400;
	LPC_SCT->EV[1].STATE = 3;
	LPC_SCT->START_L = 2;
	LPC_SCT->OUT[0].SET = 2;

	LPC_SCT->EV[2].CTRL = 0x2800;
	LPC_SCT->EV[2].STATE = 3;
	LPC_SCT->STOP_L = 4;
	LPC_SCT->OUT[0].CLR = 4;

	LPC_SCT->CTRL_H = ((ADC_PSCALE - 1) << 5) | 0x08;
	LPC_SCT->CTRL_L = ((ADC_PSCALE - 1) << 5) | 0x0A;

	Chip_ACMP_Init(LPC_CMP);
	Chip_ACMP_SetupVoltLadder(LPC_CMP, ADC_VLADDR, false);
	Chip_ACMP_EnableVoltLadder(LPC_CMP);

	volatile uint32_t delay;
	for (delay = 0; delay < 255; delay++);

	Chip_ACMP_SetPosVoltRef(LPC_CMP, ACMP_POSIN_VLO);
	Chip_ACMP_SetNegVoltRef(LPC_CMP, ACMP_NEGIN_ACMP_I1);
	Chip_Clock_DisablePeriphClock(SYSCTL_CLOCK_ACOMP);

	NVIC_EnableIRQ(SCT_IRQn);
	i2csSetup(I2C_ADDR, 0, 2);

	while (1)
		__WFI();
	return 0;
}
