/*
 * aqm0802a.c
 *
 *  Created on: 2014/10/09
 *      Author: Tetsuya Suzuki
 */

#include "chip.h"
#include "i2cm.h"
#include "mrt.h"
#include "aqm0802a.h"

#define LCD_CONT 32

void lcdCom(uint8_t com){
	uint8_t buf[3];

	buf[0] = LCD_ADRS;
	buf[1] = 0x00;
	buf[2] = com;
	i2cmTx(buf, 3);
}

void lcdDat(uint8_t dat){
	uint8_t buf[3];

	buf[0] = LCD_ADRS;
	buf[1] = 0x40;
	buf[2] = dat;
	i2cmTx(buf, 3);
}

void lcdSetup(){
	mrtWait(MRT_MS(40));//40ms
	lcdCom(0x38);//Function set + 26.3us
	lcdCom(0x39);//Function set + 26.3us
	lcdCom(0x14);//Internal OSC frequency + 26.3us
	lcdCom(0x70 | (LCD_CONT & 0x0f));//Contrast set + 26.3us
	lcdCom(0x54 | (LCD_CONT >> 4));// Power/ICON/Contrast control +26.3us
	lcdCom(0x6C);//Follower control
	mrtWait(MRT_MS(200));//200ms
	lcdCom(0x38);//Function set + 26.3us
	lcdCom(0x0C);//Display ON/OFF control + 26.3us
	lcdCom(0x01);//Clear Display
	mrtWait(MRT_US(1080));//1.08ms
}

void lcdDefChar(uint8_t code, const uint8_t *font){
	int i;

	lcdCom(0x40 | (code << 3));
	for(i = 0; i < 8; i++)
		lcdDat(font[i]);
}

void lcdLocate(uint8_t x, uint8_t y){
	lcdCom(0x80 | (x + (y > 0? 0x40 : 0)));
}

void lcdPuts(const char *s){
	while(*s)
		lcdDat(*s++);
}
