/*
 ===============================================================================
 Name        : main.c
 Author      : $(author)
 Version     :
 Copyright   : $(copyright)
 Description : main definition
 ===============================================================================
 */

#if defined (__USE_LPCOPEN)
#if defined(NO_BOARD_LIB)
#include "chip.h"
#else
#include "board.h"
#endif
#endif

#include <cr_section_macros.h>

// TODO: insert other include files here
#include "uart.h"
#include "form.h"
#include "mrt.h"
#include "i2cm.h"
#include "am2321.h"

// TODO: insert other definitions and declarations here
void resume(uint8_t adrs, ErrorCode_t err) {
	if ((adrs == AOS_ADRS) && (err == ERR_I2C_NAK)) {
		return;
	}
	while (1);
}

int main(void) {

#if defined (__USE_LPCOPEN)
#if !defined(NO_BOARD_LIB)
	// Read clock settings and update SystemCoreClock variable
	SystemCoreClockUpdate();
	// Set up and initialize all required blocks and
	// functions related to the board hardware
	Board_Init();
	// Set the LED to the state of "On"
	Board_LED_Set(0, true);
#endif
#endif

	// TODO: insert code here
	SystemCoreClockUpdate();

	Chip_Clock_EnablePeriphClock(SYSCTL_CLOCK_SWM);
	Chip_SWM_DisableFixedPin(SWM_FIXED_SWCLK);    //Disable SWCLK
	Chip_SWM_DisableFixedPin(SWM_FIXED_SWDIO);    //Disable SWDIO
	Chip_SWM_MovablePinAssign(SWM_I2C_SDA_IO, 2);    //I2C_SDA
	Chip_SWM_MovablePinAssign(SWM_I2C_SCL_IO, 3);    //I2C_SCL
	Chip_SWM_MovablePinAssign(SWM_U0_TXD_O, 4);
	Chip_SWM_MovablePinAssign(SWM_U0_RXD_I, 0);
	Chip_Clock_DisablePeriphClock(SYSCTL_CLOCK_SWM);

	uartSetup();
	mrtSetup();
	i2cmSetupErr(resume);
	i2cmSetup();

	uartPuts("Hygrometer\r\n");
	while (1) {
		int hum;
		int tmp;

		int result = aosMeasure(&hum, &tmp);
		if (result < 0) {
			uartPuts("CRC error.\r\n");
		} else {
			uartPuts("HUM:");
			uartPuts(formDec(hum, 2, 1));
			uartPuts("%, TMP:");
			uartPuts(formDec(tmp, 2, 1));
			uartPuts("*C\r\n");
		}
		mrtWait(MRT_MS(2000));    //2s
	}
	return 0;
}
