/*
===============================================================================
 Name        : main.c
 Author      : $(author)
 Version     :
 Copyright   : $(copyright)
 Description : main definition
===============================================================================
*/

#if defined (__USE_LPCOPEN)
#if defined(NO_BOARD_LIB)
#include "chip.h"
#else
#include "board.h"
#endif
#endif

#include <cr_section_macros.h>

// TODO: insert other include files here
#include "uart.h"
#include "form.h"
#include "i2cm.h"
#include "mrt.h"

// TODO: insert other definitions and declarations here
#define M24_ADRS (0x50 << 1)

void m24Write(uint16_t adrs, uint8_t data){
	uint8_t buf[4];

	buf[0] = M24_ADRS;
	buf[1] = adrs >> 8;
	buf[2] = adrs & 0xff;
	buf[3] = data;
	i2cmTx(buf, 4);
	mrtWait(MRT_MS(5));//5ms
}

uint8_t m24Read(uint16_t adrs){
	uint8_t buf[3];

	buf[0] = M24_ADRS;
	buf[1] = adrs >> 8;
	buf[2] = adrs & 0xff;
	i2cmTx(buf, 3);
	i2cmRx(buf, 2);
	return buf[1];
}

int main(void) {

#if defined (__USE_LPCOPEN)
#if !defined(NO_BOARD_LIB)
    // Read clock settings and update SystemCoreClock variable
    SystemCoreClockUpdate();
    // Set up and initialize all required blocks and
    // functions related to the board hardware
    Board_Init();
    // Set the LED to the state of "On"
    Board_LED_Set(0, true);
#endif
#endif

    // TODO: insert code here
    SystemCoreClockUpdate();

    Chip_Clock_EnablePeriphClock(SYSCTL_CLOCK_SWM);
	Chip_SWM_DisableFixedPin(SWM_FIXED_SWCLK);//Disable SWCLK
	Chip_SWM_DisableFixedPin(SWM_FIXED_SWDIO);//Disable SWDIO
 	Chip_SWM_MovablePinAssign(SWM_I2C_SDA_IO, 2);//I2C_SDA
	Chip_SWM_MovablePinAssign(SWM_I2C_SCL_IO, 3);//I2C_SCL
	Chip_SWM_MovablePinAssign(SWM_U0_TXD_O, 4);
	Chip_SWM_MovablePinAssign(SWM_U0_RXD_I, 0);
	Chip_Clock_DisablePeriphClock(SYSCTL_CLOCK_SWM);

	uartSetup();
	mrtSetup();
	i2cmSetup();

	uint16_t adrs;
	uint8_t data;

	uartPuts("EEPRM Write.\r\n");
	for(adrs = 0; adrs < 16; adrs++){
		m24Write(adrs, (uint8_t)adrs);
		uartPuts("Address:");
		uartPuts(formHex(adrs, 4));
		uartPuts(", Write:");
		uartPuts(formHex(adrs, 2));
		uartPuts("\r\n");
		mrtWait(MRT_MS(1000));//1s
	}

	uartPuts("\r\nEEPRM Read.\r\n");
	for(adrs = 0; adrs < 16; adrs++){
		data = m24Read(adrs);
		uartPuts("Address:");
		uartPuts(formHex(adrs, 4));
		uartPuts(", Read:");
		uartPuts(formHex(data, 2));
		uartPuts("\r\n");
		mrtWait(MRT_MS(1000));//1s
	}
	uartPuts("\r\nThat's all.\r\n");

	while(1) __WFI();
    return 0 ;
}
