/*
 * i2cs.c
 *
 *  Created on: 2014/10/15
 *      Author: Tetsuya Suzuki
 */

#include "chip.h"

I2C_HANDLE_T *i2csHandle;
uint32_t i2cMem[0x20];
uint8_t i2csRxBuf[16];
uint8_t i2csTxBuf[16];
void (*i2csError)(void) = NULL;
void (*i2csRxUsr)(void) = NULL;
void (*i2csTxUsr)(void) = NULL;
I2C_PARAM_T i2csRxPar;
I2C_RESULT_T i2csRxRes;
I2C_PARAM_T i2csTxPar;
I2C_RESULT_T i2csTxRes;

void i2csSetupErr(void (*pFunc)(void)) {
	i2csError = pFunc;
}

void i2csSetupRx(void (*pFunc)(void)) {
	i2csRxUsr = pFunc;
}

void i2csSetupTx(void (*pFunc)(void)) {
	i2csTxUsr = pFunc;
}

void i2csRx(void) {
	ErrorCode_t err = LPC_I2CD_API->i2c_slave_receive_intr(i2csHandle, &i2csRxPar, &i2csRxRes);
	if (err != LPC_OK) {
		if (i2csError != NULL)
			i2csError();
		else
			while (1);
	}
}

void i2csTx(void) {
	ErrorCode_t err = LPC_I2CD_API->i2c_slave_transmit_intr(i2csHandle, &i2csTxPar, &i2csTxRes);
	if (err != LPC_OK) {
		if (i2csError != NULL)
			i2csError();
		else
			while (1);
	}
}

void i2csRxCb(uint32_t err, uint32_t n) {
	i2csRx();
	if (err != LPC_OK) {
		if (i2csError != NULL)
			i2csError();
		else
			while (1);
	}
	if (i2csRxUsr != NULL)
		i2csRxUsr();
}

void i2csTxCb(uint32_t err, uint32_t n) {
	i2csTx();
	if (err != LPC_OK) {
		if (i2csError != NULL)
			i2csError();
		else
			while (1);
	}
	if (i2csTxUsr != NULL)
		i2csTxUsr();
}

void I2C_IRQHandler(void) {
	LPC_I2CD_API->i2c_isr_handler(i2csHandle);
}

void i2csSetup(uint32_t addr, uint32_t rxn, uint32_t txn) {
	i2csRxPar.num_bytes_rec = rxn;
	i2csRxPar.buffer_ptr_rec = i2csRxBuf;
	i2csRxPar.func_pt = i2csRxCb;

	i2csTxPar.num_bytes_rec = 1;
	i2csTxPar.num_bytes_send = txn;
	i2csTxPar.buffer_ptr_rec = i2csRxBuf;
	i2csTxPar.buffer_ptr_send = i2csTxBuf;
	i2csTxPar.func_pt = i2csTxCb;

	Chip_I2C_Init(LPC_I2C);
	i2csHandle = LPC_I2CD_API->i2c_setup(LPC_I2C_BASE, i2cMem);
	if (i2csHandle == NULL) {
		if (i2csError != NULL)
			i2csError();
		else
			while (1);
	}

	ErrorCode_t err = LPC_I2CD_API->i2c_set_slave_addr(i2csHandle, addr, 0xff);
	if (err != LPC_OK) {
		if (i2csError != NULL)
			i2csError();
		else
			while (1);
	}

	NVIC_EnableIRQ(I2C_IRQn);
	i2csRx();
	i2csTx();
}
