/*
===============================================================================
 Name        : hum810.c
 Author      : $(author)
 Version     :
 Copyright   : $(copyright)
 Description : main definition
===============================================================================
*/

#if defined (__USE_LPCOPEN)
#if defined(NO_BOARD_LIB)
#include "chip.h"
#else
#include "board.h"
#endif
#endif

#include <cr_section_macros.h>

// TODO: insert other include files here
#include "i2cs.h"
#include "mrt.h"

// TODO: insert other definitions and declarations here
#define I2C_ADDR (0x24 << 1)

void selLange(){
	uint8_t lange;
	static const uint32_t sel[] = {
		0b11000, 0b10100, 0b01100
	};

	lange = i2csRxBuf[1];
	if(lange > 2)
		lange = 2;
	Chip_GPIO_SetMaskedPortValue(LPC_GPIO_PORT, 0, sel[lange]);
}

void getFreq(){
	uint32_t freq;

	Chip_SCT_SetControl(LPC_SCT, SCT_CTRL_HALT_L);// stop
	freq = LPC_SCT->COUNT_U;
	LPC_SCT->COUNT_U = 0;
	Chip_SCT_ClearControl(LPC_SCT, SCT_CTRL_HALT_L);// start
	__disable_irq();
	i2csTxBuf[0] = (freq >> 24);
	i2csTxBuf[1] = (freq >> 16) & 0xff;
	i2csTxBuf[2] = (freq >> 8) & 0xff;
	i2csTxBuf[3] = freq & 0xff;
	__enable_irq();
}

int main(void) {

#if defined (__USE_LPCOPEN)
#if !defined(NO_BOARD_LIB)
    // Read clock settings and update SystemCoreClock variable
    SystemCoreClockUpdate();
    // Set up and initialize all required blocks and
    // functions related to the board hardware
    Board_Init();
    // Set the LED to the state of "On"
    Board_LED_Set(0, true);
#endif
#endif

    // TODO: insert code here
	SystemCoreClockUpdate();

	//Pin assign
	Chip_Clock_EnablePeriphClock(SYSCTL_CLOCK_SWM);// Enable the clock to the Switch Matrix
	Chip_SWM_DisableFixedPin(SWM_FIXED_SWCLK);// Disable SWCLK
	Chip_SWM_DisableFixedPin(SWM_FIXED_SWDIO);// Disable SWDIO
   	Chip_SWM_DisableFixedPin(SWM_FIXED_RST);// Disable RESET
 	Chip_SWM_MovablePinAssign(SWM_I2C_SDA_IO, 0);//I2C_SDA
	Chip_SWM_MovablePinAssign(SWM_I2C_SCL_IO, 5);//I2C_SCL
	Chip_SWM_MovablePinAssign(SWM_CTIN_0_I, 1);
	Chip_Clock_DisablePeriphClock(SYSCTL_CLOCK_SWM);// Disable the clock to the Switch Matrix to save power

	Chip_Clock_EnablePeriphClock(SYSCTL_CLOCK_IOCON);
	Chip_IOCON_PinSetMode(LPC_IOCON, IOCON_PIO2, PIN_MODE_INACTIVE);
	Chip_IOCON_PinSetMode(LPC_IOCON, IOCON_PIO3, PIN_MODE_INACTIVE);
	Chip_IOCON_PinSetMode(LPC_IOCON, IOCON_PIO4, PIN_MODE_INACTIVE);
	Chip_IOCON_PinEnableOpenDrainMode(LPC_IOCON, IOCON_PIO2);
	Chip_IOCON_PinEnableOpenDrainMode(LPC_IOCON, IOCON_PIO3);
	Chip_IOCON_PinEnableOpenDrainMode(LPC_IOCON, IOCON_PIO4);
	Chip_Clock_DisablePeriphClock(SYSCTL_CLOCK_IOCON);

	//GPIO setup
	Chip_Clock_EnablePeriphClock(SYSCTL_CLOCK_GPIO);
	Chip_GPIO_SetPinDIROutput(LPC_GPIO_PORT, 0, 2);
	Chip_GPIO_SetPinDIROutput(LPC_GPIO_PORT, 0, 3);
	Chip_GPIO_SetPinDIROutput(LPC_GPIO_PORT, 0, 4);
	Chip_GPIO_SetPortMask(LPC_GPIO_PORT, 0, ~(0x1c));

	//SCT setup
	Chip_SCT_Init(LPC_SCT);//clocks and reset
	Chip_SCT_Config(LPC_SCT, SCT_CONFIG_32BIT_COUNTER | SCT_CONFIG_CLKMODE_INCLK);
	Chip_SCT_ClearControl(LPC_SCT, SCT_CTRL_HALT_L);// start

	//MRT setup
	mrtSetup();
	mrtRepeat(SystemCoreClock, getFreq);
	mrtRepeatStart();

	//I2C setup
	i2csRxBuf[1] = 1;
	selLange(1);
	i2csSetupRx(selLange);
	i2csSetup(I2C_ADDR, 2, 4);

	while(1) __WFI();// wait for interrupt
    return 0 ;
}
