/*
 * uart.c
 *
 *  Created on: 2014/10/11
 *      Author: Tetsuya Suzuki
 */

#include "chip.h"
#include <string.h>

UART_HANDLE_T *uartHandle;
uint32_t uartMem[0x10];
void (*uartError)(void) = NULL;

void uartSetupErr(void (*pFunc)(void)){
	uartError = pFunc;
}

void uartSetup(){
	Chip_Clock_SetUARTClockDiv(1);
	Chip_UART_Init(LPC_USART0);

	uartHandle = LPC_UARTD_API->uart_setup((uint32_t) LPC_USART0, (uint8_t *) &uartMem);
	if (uartHandle == NULL) {
		if(uartError == NULL)
			while(1);
		else
			uartError();
	}

	UART_CONFIG_T cfg;
	cfg.sys_clk_in_hz = SystemCoreClock;
	cfg.baudrate_in_hz = 9600;
	cfg.config = 1;
	cfg.sync_mod = 0;
	cfg.error_en = NO_ERR_EN;

	uint32_t ret = LPC_UARTD_API->uart_init(uartHandle, &cfg);
	if (ret) {
		Chip_SYSCTL_SetUSARTFRGDivider(0xff);
		Chip_SYSCTL_SetUSARTFRGMultiplier(ret);
	}
}

inline uint8_t uartGetch(){
	return LPC_UARTD_API->uart_get_char(uartHandle);
}

inline void uartPutch(uint8_t c){
	LPC_UARTD_API->uart_put_char(uartHandle, c);
}

void uartPuts(const char *s){
	UART_PARAM_T par;

	par.buffer = (uint8_t *) s;
	par.size = strlen(s);
	par.transfer_mode = TX_MODE_SZERO;
	par.driver_mode = DRIVER_MODE_POLLING;

	uint32_t ret = LPC_UARTD_API->uart_put_line(uartHandle, &par);
	if(ret){
		if(uartError == NULL)
			while(1);
		else
			uartError();
	}
}
