/*
 * remocon.c
 *
 *  Created on: 2014/10/25
 *      Author: Tetsuya Suzuki
 */
#include "chip.h"
#include "mrt.h"
#include "remocon.h"

uint8_t rc_irport;

void rc_setup(uint8_t irport) {
	rc_irport = irport;

	Chip_MRT_SetMode(LPC_MRT_CH1, MRT_MODE_ONESHOT);

	Chip_SCTPWM_Init(LPC_SCT);
	Chip_SCTPWM_SetRate(LPC_SCT, 38000);
	Chip_SCTPWM_SetDutyCycle(LPC_SCT, 1, Chip_SCTPWM_PercentageToTicks(LPC_SCT, 50));
	Chip_SCTPWM_SetOutPin(LPC_SCT, 1, 0);
}

#define RC_ROFF LPC_GPIO_PORT->B[0][rc_irport]
#define RC_RON !RC_ROFF
#define RC_COUNT LPC_MRT_CH1->TIMER

int rc_receive(irform_t *ir) {
	uint8_t i, j = 0; // loop counter
	uint32_t lt, lh, ll;
	uint32_t st, sh, sl;
	uint32_t son, soff;
	uint32_t told; // old lap time
	uint32_t tnew; // lap time
	uint32_t repeat;

	lt = lh = ll = 0;
	st = sh = sl = 0xffffffff;
	ir->multi = false;

	//MRT setup
	Chip_MRT_SetInterval(LPC_MRT_CH1, 0x7fffffff | MRT_INTVAL_LOAD);

	//Count leader on length
	told = RC_COUNT;
	while ((told - RC_COUNT) < RC_DTCT) {
		if (RC_COUNT < RC_CLKS_S)
			return RC_ERR_TIMEOUT; //time out
		if (RC_ROFF)
			told = RC_COUNT;
	}
	while (RC_RON)
		;
	tnew = RC_COUNT;
	ir->lon = told - tnew;
	told = tnew;

	//Count leader off length
	while (RC_ROFF)
		if ((told - RC_COUNT) > RC_TERM)
			return RC_ERR_ILLEGAL;
	tnew = RC_COUNT;
	ir->loff = told - tnew;
	told = tnew;

	// signal analyze
	for (i = 0; i < RC_SIGLEN * 8; i++) {

		//Count trail length
		while (RC_RON)
			;
		tnew = RC_COUNT;
		son = told - tnew;
		told = tnew;

		if (son > lt)
			lt = son;
		else if (son < st)
			st = son;

		//Decode signal
		while (RC_ROFF)
			if ((told - RC_COUNT) > RC_TERM)
				goto EXIT1;
		tnew = RC_COUNT;
		soff = told - tnew;
		told = tnew;

		j = i / 8;
		ir->code1[j] <<= 1;
		if (soff > son * 2) { //case 1
			ir->code1[j] |= 1;
			if (soff > lh)
				lh = soff;
			else if (soff < sh)
				sh = soff;
		} else { //case 0
			if (soff > ll)
				ll = soff;
			else if (soff < sl)
				sl = soff;
		}
	}
	return RC_ERR_TOOLONG; //too long
	EXIT1: if (i % 8)
		return RC_ERR_NOTFIT;
	ir->count1 = j + 1;
	ir->t = (lt + st) / 2;
	ir->h = (lh + sh) / 2;
	ir->l = (ll + sl) / 2;

	//search next signal
	while (RC_ROFF)
		if ((told - RC_COUNT) > RC_CONT)
			return RC_ERR_NORMAL;

	//find next signal
	tnew = RC_COUNT;
	ir->gap = told - tnew;
	told = tnew;

	//pass reader on
	while (RC_RON)
		;
	told = RC_COUNT;

	//pass leader off
	while (RC_ROFF)
		if ((told - RC_COUNT) > RC_TERM)
			return RC_ERR_ILLEGAL;
	tnew = RC_COUNT;
	repeat = told - tnew;
	told = tnew;

	//case repeat
	if ((repeat < ir->loff * 8 / 10) || (repeat > ir->loff * 12 / 10))
		return RC_ERR_NORMAL;

	ir->multi = true;

	//Signal decode
	for (i = 0; i < RC_SIGLEN * 8; i++) {
		//Count trail length
		while (RC_RON)
			;
		tnew = RC_COUNT;
		son = told - tnew;
		told = tnew;

		//Decode signal
		while (RC_ROFF)
			if ((told - RC_COUNT) > RC_TERM)
				goto EXIT2;
		tnew = RC_COUNT;
		soff = told - tnew;
		told = tnew;
		j = i / 8;
		ir->code2[j] <<= 1;
		ir->code2[j] |= (son * 2 < soff);
	}
	return RC_ERR_TOOLONG; //too long
	EXIT2: if (i % 8)
		return RC_ERR_NOTFIT;
	ir->count2 = j + 1;
	return RC_ERR_NORMAL;
}

inline void rc_tstop(){
	Chip_SCTPWM_Stop(LPC_SCT);
	LPC_SCT->OUTPUT = 0;
}

void rc_transmit(irform_t *ir) {
	int i, j; // loop counter
	uint8_t code;

	Chip_SCTPWM_Start(LPC_SCT);// leader on
	mrtSleep(ir->lon);
	rc_tstop();// leader off
	mrtSleep(ir->loff);

	for (i = 0; i < ir->count1; i++) {
		code = ir->code1[i];
		for (j = 0; j < 8; j++) {
			Chip_SCTPWM_Start(LPC_SCT);// on
			mrtSleep(ir->t);
			rc_tstop();// off
			mrtSleep(code & 0x80 ? ir->h : ir->l);
			code <<= 1;
		}
	}
	Chip_SCTPWM_Start(LPC_SCT);// on
	mrtSleep(ir->t);
	rc_tstop();// off

	if (!ir->multi)
		return;

	mrtSleep(ir->gap);//gap

	Chip_SCTPWM_Start(LPC_SCT);// leader on
	mrtSleep(ir->lon);
	rc_tstop();// leader off
	mrtSleep(ir->loff);

	for (i = 0; i < ir->count2; i++) {
		code = ir->code2[i];
		for (j = 0; j < 8; j++) {
			Chip_SCTPWM_Start(LPC_SCT);// on
			mrtSleep(ir->t);
			rc_tstop();// off
			mrtSleep(code & 0x80 ? ir->h : ir->l);
			code <<= 1;
		}
	}
	Chip_SCTPWM_Start(LPC_SCT);// on
	mrtSleep(ir->t);
	rc_tstop();// off
}

int rc_ircmp(irform_t *ir1, irform_t *ir2) {
	int i; // loop counter

	if ((ir1->count1 != ir2->count1) ||
		(ir1->multi != ir2->multi))
		return -1;

	for (i = 0; i < ir1->count1; i++)
		if (ir1->code1[i] != ir2->code1[i])
			return -1;

	if (!ir1->multi)
		return 0;

	if (ir1->count2 != ir2->count2)
		return -1;

	for (i = 0; i < ir1->count2; i++)
		if (ir1->code2[i] != ir2->code2[i])
			return -1;

	return 0;
}
