/*
 ===============================================================================
 Name        : irt810.c
 Author      : $(author)
 Version     :
 Copyright   : $(copyright)
 Description : main definition
 ===============================================================================
 */

#if defined (__USE_LPCOPEN)
#if defined(NO_BOARD_LIB)
#include "chip.h"
#else
#include "board.h"
#endif
#endif

#include <cr_section_macros.h>

// TODO: insert other include files here
#include "mrt.h"
#include "remocon.h"

// TODO: insert other definitions and declarations here
volatile int button;

void PININT2_IRQHandler(void) {
	Chip_PININT_ClearIntStatus(LPC_PININT, PININTCH2);
	button = 0;
}

void PININT3_IRQHandler(void) {
	Chip_PININT_ClearIntStatus(LPC_PININT, PININTCH3);
	button = 1;
}

void PININT4_IRQHandler(void) {
	Chip_PININT_ClearIntStatus(LPC_PININT, PININTCH4);
	button = 2;
}

int main(void) {

#if defined (__USE_LPCOPEN)
#if !defined(NO_BOARD_LIB)
	// Read clock settings and update SystemCoreClock variable
	SystemCoreClockUpdate();
	// Set up and initialize all required blocks and
	// functions related to the board hardware
	Board_Init();
	// Set the LED to the state of "On"
	Board_LED_Set(0, true);
#endif
#endif

	// TODO: insert code here
	static uint8_t data[3][6] = {
		{ 0x55, 0x5a, 0xf1, 0x48, 0xc0, 0xcd },//3
		{ 0x55, 0x5a, 0xf1, 0x48, 0x40, 0xc5 },//2
		{ 0x55, 0x5a, 0xf1, 0x48, 0x80, 0xc9 } //1
	};
	irform_t ir;

	ir.lon = RC_CLKS_US(3372);
	ir.loff = RC_CLKS_US(1683);
	ir.t = RC_CLKS_US(416);
	ir.h = RC_CLKS_US(1256);
	ir.l = RC_CLKS_US(420);
	ir.multi = false;
	ir.count1 = 6;

	SystemCoreClockUpdate();

	//setup pins
	Chip_Clock_EnablePeriphClock(SYSCTL_CLOCK_SWM);
	Chip_SWM_DisableFixedPin(SWM_FIXED_SWCLK);    // Disable SWCLK
	Chip_SWM_DisableFixedPin(SWM_FIXED_SWDIO);    // Disable SWDIO
	Chip_SWM_MovablePinAssign(SWM_CTOUT_0_O, 0);
	Chip_Clock_DisablePeriphClock(SYSCTL_CLOCK_SWM);

	Chip_Clock_EnablePeriphClock(SYSCTL_CLOCK_IOCON);
	Chip_IOCON_PinSetMode(LPC_IOCON, IOCON_PIO0, PIN_MODE_PULLDN);    //Ir LED
	Chip_IOCON_PinSetMode(LPC_IOCON, IOCON_PIO2, PIN_MODE_PULLUP); //Switch Yellow
	Chip_IOCON_PinSetMode(LPC_IOCON, IOCON_PIO3, PIN_MODE_PULLUP); //Switch Green
	Chip_IOCON_PinSetMode(LPC_IOCON, IOCON_PIO4, PIN_MODE_PULLUP);  //Switch Red
	Chip_IOCON_PinEnableHysteresis(LPC_IOCON, IOCON_PIO2);
	Chip_IOCON_PinEnableHysteresis(LPC_IOCON, IOCON_PIO3);
	Chip_IOCON_PinEnableHysteresis(LPC_IOCON, IOCON_PIO4);
	Chip_Clock_DisablePeriphClock(SYSCTL_CLOCK_IOCON);

	Chip_Clock_EnablePeriphClock(SYSCTL_CLOCK_GPIO);
	Chip_GPIO_SetPinDIRInput(LPC_GPIO_PORT, 0, 2);    //Switch Yellow
	Chip_GPIO_SetPinDIRInput(LPC_GPIO_PORT, 0, 3);    //Switch Green
	Chip_GPIO_SetPinDIRInput(LPC_GPIO_PORT, 0, 4);    //Switch Red

	Chip_SYSCTL_SetPinInterrupt(2, 2);
	Chip_SYSCTL_SetPinInterrupt(3, 3);
	Chip_SYSCTL_SetPinInterrupt(4, 4);
	Chip_PININT_SetPinModeEdge(LPC_PININT, PININTCH2 | PININTCH3 | PININTCH4);
	Chip_PININT_EnableIntLow(LPC_PININT, PININTCH2 | PININTCH3 | PININTCH4);

	mrtSetup();
	mrtSetupSleep();
	rc_setup(6);
	NVIC_EnableIRQ(PININT2_IRQn);
	NVIC_EnableIRQ(PININT3_IRQn);
	NVIC_EnableIRQ(PININT4_IRQn);
	button = -1;

	while (1) {
		__WFI();
		if (button > -1) {
			ir.code1 = data[button];
			rc_transmit(&ir);
			button = -1;
		}
	}
	return 0;
}
