/*
===============================================================================
 Name        : main.c
 Author      : $(author)
 Version     :
 Copyright   : $(copyright)
 Description : main definition
===============================================================================
*/

#if defined (__USE_LPCOPEN)
#if defined(NO_BOARD_LIB)
#include "chip.h"
#else
#include "board.h"
#endif
#endif

#include <cr_section_macros.h>

// TODO: insert other include files here
#include "aqm0802a.h"
#include "lps25h.h"
#include "am2321.h"
#include "form.h"
#include "mrt.h"
#include "i2cm.h"

// TODO: insert other definitions and declarations here
void resume(uint8_t adrs, ErrorCode_t err) {
	if ((adrs == AOS_ADRS) && (err == ERR_I2C_NAK)) {
		return;
	}
	while (1);
}

int main(void) {

#if defined (__USE_LPCOPEN)
#if !defined(NO_BOARD_LIB)
    // Read clock settings and update SystemCoreClock variable
    SystemCoreClockUpdate();
    // Set up and initialize all required blocks and
    // functions related to the board hardware
    Board_Init();
    // Set the LED to the state of "On"
    Board_LED_Set(0, true);
#endif
#endif

    // TODO: insert code here
	SystemCoreClockUpdate();

	//Pin assign
	Chip_Clock_EnablePeriphClock(SYSCTL_CLOCK_SWM);
	Chip_SWM_DisableFixedPin(SWM_FIXED_SWCLK);// Disable SWCLK
	Chip_SWM_DisableFixedPin(SWM_FIXED_SWDIO);// Disable SWDIO
 	Chip_SWM_MovablePinAssign(SWM_I2C_SDA_IO, 0);// I2C_SDA
	Chip_SWM_MovablePinAssign(SWM_I2C_SCL_IO, 1);// I2C_SCL
 	Chip_Clock_DisablePeriphClock(SYSCTL_CLOCK_SWM);

	const uint8_t squar[] = {
		0x04, 0x0a, 0x02, 0x04, 0x0e, 0x00, 0x00, 0x00
	};
	const uint8_t heart[] = {
		0x00, 0x0a, 0x1f, 0x1f, 0x1f, 0x0e, 0x04, 0x00
	};
	const uint8_t sheart[] = {
		0x00, 0x0a, 0x1f, 0x1f, 0x0e, 0x04, 0x00, 0x00
	};
	const uint8_t degree[] = {
		0x07, 0x05, 0x07, 0x00, 0x00, 0x00, 0x00, 0x00
	};

	mrtSetup();
	i2cmSetupErr(resume);
	i2cmSetup();
	lpsSetup();

	lcdSetup();
	lcdDefChar(0, squar);
	lcdDefChar(1, heart);
	lcdDefChar(2, sheart);
	lcdDefChar(3, degree);

	lcdLocate(0, 0);
	lcdDat('I');
	lcdDat(0);
	lcdPuts("C LCD");
	lcdDat(1);
	lcdLocate(0, 1);
	lcdPuts("AQM0802A");

	int count = 0;
	while(1){
		float atm, dum;
		int hum, tmp;

		mrtWait(MRT_MS(2000));
		lpsMeasure(&atm, &dum);
		aosMeasure(&hum, &tmp);

		tmp = (tmp + ((tmp < 0) ? -5: 5)) / 10;
		hum = (hum + 5) / 10;

		lcdLocate(0, 0);
		lcdPuts(formDec(tmp, 2, 0));
		lcdDat(3);
		lcdPuts("C ");
		lcdPuts(formDec(hum, 2, 0));
		lcdPuts("%");
		lcdLocate(0, 1);
		lcdPuts(formFloat(atm, 4, 0));
		lcdPuts("hPa");
		lcdDat((count++ & 1) ? 1: 2);
	}
	return 0 ;
}
